/*
 * (c) Copyright 2015 -- Anders Torger
 *
 * This program is open source. For license terms, see the LICENSE file.
 *
 */
#ifndef INTERP_H_
#define INTERP_H_

static inline double
curve_y(double x,
        const double tc[],
        const int tc_len)
{
    double xm = x * (tc_len-1);
    int idx = (int)xm;
    if (idx >= tc_len-1) {
        return tc[tc_len-1];
    }
    if (idx < 0) {
        return tc[0];
    }
    double d = xm - (double)idx; // [0 .. 1]
    return (1.0 - d) * tc[idx] + d * tc[idx+1];
}

static inline double
curve_y_rng(double x,
            double range[2],
            const double tc[],
            const int tc_len)
{
    double xm = (x - range[0]) / (range[1] - range[0]) * (tc_len - 1);
    if (xm <= 0) {
        return tc[0];
    }
    int idx = (int)xm;
    if (idx >= tc_len-1) {
        return tc[tc_len-1];
    }
    double d = xm - (double)idx; // [0 .. 1]
    return (1.0 - d) * tc[idx] + d * tc[idx+1];
}

double
inverse_curve(double y,
              const double tc[],
              int tc_len);

void
cubic_spline(const double x[],
             const double y[],
             int len,
             const double out_x[],
             double out_y[],
             int out_len);

void
rounded_linear_interpolation(const double x[],
                             const double y[],
                             const int n,
                             const double out_x[],
                             double out_y[],
                             const int out_n);

void
linear_interpolation(const double x[],
                     const double y[],
                     const int n,
                     const double out_x[],
                     double out_y[],
                     const int out_n);

void
reconstruct_tonecurve(double tc[],
                      int tc_len);

bool
point_inside_polygon(const double vertx[],
                     const double verty[],
                     int nvert,
                     double testx,
                     double testy);

#endif
